@@ -26,15 +26,7 @@
 #include <string.h>
 #include "frotz.h"
 
-#ifdef MSDOS_16BIT
-
-#include <alloc.h>
-
-#define malloc(size)	farmalloc (size)
-#define realloc(size,p)	farrealloc (size,p)
-#define free(size)	farfree (size)
-
-#else
+#ifndef MSDOS_16BIT
 
 #include <stdlib.h>
 
@@ -43,9 +35,14 @@
 #define SEEK_CUR 1
 #define SEEK_END 2
 #endif
+#endif /* !MSDOS_16BIT */
 
-#define far
-
+#ifdef __WATCOMC__
+zbyte huge *zmp = NULL;
+zbyte huge *pcp = NULL;
+#else
+zbyte *zmp = NULL;
+zbyte *pcp = NULL;
 #endif
 
 extern void seed_random (int);
@@ -70,9 +67,6 @@ extern void (*var_opcodes[]) (void);
 /* char save_name[MAX_FILE_NAME + 1] = DEFAULT_SAVE_NAME; */
 char auxilary_name[MAX_FILE_NAME + 1] = DEFAULT_AUXILARY_NAME;
 
-zbyte far *zmp = NULL;
-zbyte far *pcp = NULL;
-
 static FILE *story_fp = NULL;
 
 /*
@@ -81,7 +75,6 @@ static FILE *story_fp = NULL;
  * Nitfol interpreter.
  * Undo blocks are stored as differences between states.
  */
-
 typedef struct undo_struct undo_t;
 struct undo_struct {
 	undo_t *next;
@@ -94,12 +87,26 @@ struct undo_struct {
 	/* undo diff and stack data follow */
 };
 
-static undo_t *first_undo = NULL, *last_undo = NULL, *curr_undo = NULL;
-static zbyte *prev_zmp, *undo_diff;
+static undo_t huge *first_undo = NULL, huge *last_undo = NULL,
+	      huge *curr_undo = NULL;
+static zbyte huge *prev_zmp, *undo_diff;
 
 static int undo_count = 0;
 
 
+#ifdef __WATCOMC__
+void huge *zrealloc(void huge *p, long size, size_t old_size)
+{
+	void huge *q = zmalloc(size);
+	if (q == NULL)
+		return NULL;
+	_fmemcpy(q, p, size < old_size ? size : old_size);
+	zfree(p);
+	return q;
+}
+#endif /* __WATCOMC__ */
+
+
 /*
  * get_header_extension
  *
@@ -157,6 +164,45 @@ void restart_header(void)
 	SET_BYTE(H_CONFIG, z_header.config);
 	SET_WORD(H_FLAGS, z_header.flags);
 
+	if (z_header.version == V6 && f_setup.interpreter_number == 0) {
+		if (story_id == JOURNEY) {
+			switch (z_header.release) {
+			case 26: f_setup.interpreter_number = INTERP_MACINTOSH; break;
+			case 30: f_setup.interpreter_number = INTERP_AMIGA; break;
+			case 77: f_setup.interpreter_number = INTERP_APPLE_IIE; break;
+			case 83: f_setup.interpreter_number = INTERP_MSDOS; break;
+			default: f_setup.interpreter_number = INTERP_AMIGA; break;
+			}
+			z_header.interpreter_number = f_setup.interpreter_number;
+		} else if (story_id == SHOGUN) {
+			switch (z_header.release) {
+			case 292: f_setup.interpreter_number = INTERP_MACINTOSH; break;
+			case 295: f_setup.interpreter_number = INTERP_AMIGA; break;
+			case 311: f_setup.interpreter_number = INTERP_APPLE_IIE; break;
+			case 322: f_setup.interpreter_number = INTERP_MSDOS; break;
+			default: f_setup.interpreter_number = INTERP_AMIGA; break;
+			}
+			z_header.interpreter_number = f_setup.interpreter_number;
+		} else if (story_id == ZORK_ZERO) {
+			switch (z_header.release) {
+			case 296: f_setup.interpreter_number = INTERP_MACINTOSH; break;
+			case 366: f_setup.interpreter_number = INTERP_AMIGA; break;
+			case 383: f_setup.interpreter_number = INTERP_APPLE_IIE; break;
+			case 393: f_setup.interpreter_number = INTERP_MSDOS; break;
+			default: f_setup.interpreter_number = INTERP_AMIGA; break;
+			}
+			z_header.interpreter_number = f_setup.interpreter_number;
+		} else if (story_id == ARTHUR) {
+			switch (z_header.release) {
+			case 54: f_setup.interpreter_number = INTERP_AMIGA; break;
+			case 64: f_setup.interpreter_number = INTERP_APPLE_IIE; break;
+			case 74: f_setup.interpreter_number = INTERP_MSDOS; break;
+			default: f_setup.interpreter_number = INTERP_AMIGA; break;
+			}
+			z_header.interpreter_number = f_setup.interpreter_number;
+		}
+        }
+
 	if (z_header.version >= V4) {
 		SET_BYTE(H_INTERPRETER_NUMBER, z_header.interpreter_number);
 		SET_BYTE(H_INTERPRETER_VERSION, z_header.interpreter_version);
@@ -307,6 +353,7 @@ void init_memory(void)
 		{	SORCERER,  87, "860904" },
 		{   SPELLBREAKER,  63, "850916" },
 		{   SPELLBREAKER,  87, "860904" },
+		{     PLANETFALL,   1, "830517" },
 		{     PLANETFALL,  20, "830708" },
 		{     PLANETFALL,  26, "831014" },
 		{     PLANETFALL,  29, "840118" },
@@ -410,7 +457,7 @@ void init_memory(void)
 		os_fatal("Cannot open story file");
 
 	/* Allocate memory for story header */
-	if ((zmp = (zbyte far *) malloc(64)) == NULL)
+	if ((zmp = (zbyte huge *) zmalloc(64)) == NULL)
 		os_fatal("Out of memory");
 
 	/* Load header into memory */
@@ -488,7 +535,7 @@ void init_memory(void)
 	}
 
 	/* Allocate memory for story data */
-	if ((zmp = (zbyte far *) realloc(zmp, story_size)) == NULL)
+	if ((zmp = (zbyte huge *) zrealloc(zmp, story_size, 64)) == NULL)
 		os_fatal("Out of memory");
 
 	/* Load story file in chunks of 32KB */
@@ -517,30 +564,30 @@ void init_memory(void)
  */
 void init_undo(void)
 {
-	void far *reserved;
+	void huge *reserved;
 
 	reserved = NULL;	/* makes compilers shut up */
 
 	if (reserve_mem != 0) {
-		if ((reserved = malloc(reserve_mem)) == NULL)
+		if ((reserved = zmalloc(reserve_mem)) == NULL)
 			return;
 	}
 
 	/* Allocate z_header.dynamic_size bytes for previous dynamic
 	 * zmp state + 1.5 z_header.dynamic_size for Quetzal diff + 2.
 	 */
-	prev_zmp = malloc(z_header.dynamic_size);
-	undo_diff = malloc(((unsigned long)z_header.dynamic_size * 3) / 2 + 2);
+	prev_zmp = zmalloc(z_header.dynamic_size);
+	undo_diff = zmalloc(((unsigned long)z_header.dynamic_size * 3) / 2 + 2);
 	if ((undo_diff != NULL) && (prev_zmp != NULL)) {
 		memmove (prev_zmp, zmp, z_header.dynamic_size);
 	} else {
 		f_setup.undo_slots = 0;
-		if (prev_zmp != NULL) free(prev_zmp);
-		if (undo_diff != NULL) free(undo_diff);
+		if (prev_zmp != NULL) zfree(prev_zmp);
+		if (undo_diff != NULL) zfree(undo_diff);
 	}
 
 	if (reserve_mem != 0)
-		free(reserved);
+		zfree(reserved);
 } /* init_undo */
 
 
@@ -561,7 +608,7 @@ static void free_undo(int count)
 		if (curr_undo == first_undo)
 			curr_undo = curr_undo->next;
 		first_undo = first_undo->next;
-		free (p);
+		zfree (p);
 		undo_count--;
 	}
 	if (first_undo)
@@ -585,8 +632,8 @@ void reset_memory(void)
 
 	if (undo_diff) {
 		free_undo(undo_count);
-		free(undo_diff);
-		free(prev_zmp);
+		zfree(undo_diff);
+		zfree(prev_zmp);
 	}
 
 	undo_diff = NULL;
@@ -594,7 +641,7 @@ void reset_memory(void)
 	prev_zmp = NULL;
 
 	if (zmp)
-		free(zmp);
+		zfree(zmp);
 	zmp = NULL;
 } /* reset_memory */
 
@@ -753,17 +800,17 @@ void z_restore(void)
 		/* Close auxilary file */
 		fclose (gfp);
 	} else {
-
-//	long pc;
-//	zword release;
-//	zword addr;
-//	int i;
-
+	/*
+	 * long pc;
+	 * zword release;
+	 * zword addr;
+	 * int i;
+	 */
 		/* Get the file name */
 		new_name = os_read_file_name(f_setup.save_name, FILE_RESTORE);
 		if (new_name == NULL)
 			goto finished;
-		free(f_setup.save_name);
+		zfree(f_setup.save_name);
 		f_setup.save_name = strdup(new_name);
 
 		/* Open game file */
@@ -980,13 +1027,13 @@ void z_save(void)
 		fclose(gfp);
 
 	} else {
-
-//	long pc;
-//	zword addr;
-//	zword nsp, nfp;
-//	int skip;
-//	int i;
-
+	/*
+	 * long pc;
+	 * zword addr;
+	 * zword nsp, nfp;
+	 * int skip;
+	 * int i;
+	 */
 		/* Get the file name */
 		new_name = os_read_file_name(f_setup.save_name, FILE_SAVE);
 		if (new_name == NULL)
@@ -1030,7 +1077,7 @@ int save_undo(void)
 {
 	long diff_size;
 	zword stack_size;
-	undo_t *p;
+	undo_t huge *p;
 	long pc;
 
 	/* undo feature unavailable */
@@ -1055,7 +1102,7 @@ int save_undo(void)
 	diff_size = mem_diff(zmp, prev_zmp, z_header.dynamic_size, undo_diff);
 	stack_size = stack + STACK_SIZE - sp;
 	do {
-		p = malloc(sizeof (undo_t) + diff_size + stack_size * sizeof (*sp));
+		p = zmalloc(sizeof (undo_t) + diff_size + stack_size * sizeof (*sp));
 		if (p == NULL)
 			free_undo(1);
 	} while (!p && undo_count);
