@@ -9,28 +9,73 @@
 #define FROTZ_H_
 
 
+/******************************************************************************/
+/* MSDOS settings */
+/******************************************************************************/
 /* For now we assume DOS Frotz is always 16-bit */
-#ifdef __TURBOC__
-#define MSDOS_16BIT
+#if defined __TURBOC__ || defined __WATCOMC__
+  #define MSDOS_16BIT
 #endif
 
 #ifdef MSDOS_16BIT
-#include "..\dos\defs.h"
+#ifdef __TURBOC__
+#include "../dos/defs.h"
+#else
+#include "../owdos/defs.h"
+#endif
+
 #ifdef USE_UTF8
 #error UTF-8 is not supported in DOS Frotz
 #endif
+
+#include <malloc.h>
+
+#define zmalloc(size)	halloc((size), 1)
+#define zfree(p)	hfree(p)
+
+#ifdef __WATCOMC__
+/*
+ * Open Watcom C's runtime library does not have a function to reallocate
+ * "huge" memory blocks --- unlike Borland Turbo C --- so we need to
+ * implement our own function to do that.  (Luckily zrealloc is used only in
+ * one place in fastmem.c, which simplifies things somewhat.)
+ */
+#define huge		_huge
+void huge *zrealloc(void huge *p, long size, size_t old_size);
+#else
+#define zrealloc(p, size, old_size) farrealloc((p), (size))
 #endif
 
-#ifndef MSDOS_16BIT
-#include "defs.h"
-#include "git_hash.h"
-#endif
+#endif /* MSDOS_16BIT */
+/******************************************************************************/
+/******************************************************************************/
 
 #ifndef __UNIX_PORT_FILE
 #include <signal.h>
 typedef int bool;
 #endif /* __UNIX_PORT_FILE */
 
+
+/******************************************************************************/
+/* For Amiga and Unix */
+/******************************************************************************/
+#ifndef MSDOS_16BIT
+
+#include "defs.h"
+#include "git_hash.h"
+
+#define huge
+#define zmalloc(size)	malloc(size)
+#define zfree(p)	free(p)
+#define zrealloc(p, size, old_size) realloc((p), (size))
+
+#endif /* !MSDOS_16BIT */
+/******************************************************************************/
+/******************************************************************************/
+
+/******************************************************************************/
+/* For everything */
+/******************************************************************************/
 #include <stdio.h>
 
 #ifndef TRUE
@@ -54,7 +99,8 @@ typedef int bool;
 #endif /* !PATH_MAX */
 
 
-#if defined(WIN32) || defined(_WIN32) || defined(WIN64) || defined(_WIN64) || defined (__CYGWIN__)
+/* MSDOS and derived systems use a backslash for path separator */
+#if defined(MSDOS_16BIT) || defined(WIN32) || defined(_WIN32) || defined(WIN64) || defined(_WIN64) || defined (__CYGWIN__)
 #define PATH_SEPARATOR '\\'
 #else
 #define PATH_SEPARATOR '/'
@@ -366,7 +412,6 @@ extern const char build_timestamp[];
 #define ZC_LATIN1_MAX 0xff
 
 /*** File types ***/
-
 #define FILE_RESTORE 0
 #define FILE_SAVE 1
 #define FILE_SCRIPT 2
@@ -376,11 +421,14 @@ extern const char build_timestamp[];
 #define FILE_SAVE_AUX 6
 
 /*** Data access macros ***/
-
 #define SET_BYTE(addr,v)  { zmp[addr] = v; }
 #define LOW_BYTE(addr,v)  { v = zmp[addr]; }
 #define CODE_BYTE(v)	  { v = *pcp++;    }
 
+
+/******************************************************************************/
+/* Amiga macros */
+/******************************************************************************/
 #if defined (AMIGA)
 
 extern zbyte *pcp;
@@ -396,15 +444,42 @@ extern zbyte *zmp;
 #define GET_PC(v)         { v = pcp - zmp; }
 #define SET_PC(v)         { pcp = zmp + v; }
 
-#endif
+#endif /* AMIGA */
+/******************************************************************************/
+/******************************************************************************/
 
+
+/******************************************************************************/
+/* MSDOS macros */
+/******************************************************************************/
 #if defined (MSDOS_16BIT)
-extern zbyte *pcp;
-extern zbyte *zmp;
 
 #define lo(v)   ((zbyte *)&v)[0]
 #define hi(v)   ((zbyte *)&v)[1]
 
+#ifdef __WATCOMC__
+
+extern zbyte _huge *pcp;
+extern zbyte _huge *zmp;
+
+zword bswap16(zword x);
+#pragma aux bswap16 = "xchg ah, al" parm [ax] value [ax];
+
+/*
+ * TODO: make these more efficient (and still correct).
+ */
+#define SET_WORD(addr, v)	{ *(zword _huge *)(zmp+(addr))=bswap16(v); }
+#define LOW_WORD(addr, v)	{ (v)=bswap16(*(zword _huge *)(zmp+(addr))); }
+#define HIGH_WORD(addr, v)	{ (v)=bswap16(*(zword _huge *)(zmp+(addr))); }
+#define CODE_WORD(v)		{ (v)=bswap16(*(zword _huge *)pcp); pcp+=2; }
+#define GET_PC(v)		{ (v) = pcp - zmp; }
+#define SET_PC(v)		{ pcp = zmp + (v); }
+
+#else /* !__WATCOMC__ */
+
+extern zbyte *pcp;
+extern zbyte *zmp;
+
 /*
  * Turbo C has a strange limitation with passing members of structs to
  * assembly code within a macro.  If more than one struct have members
@@ -417,7 +492,6 @@ extern zbyte *zmp;
  * struct members.
  *
  */
-
 #define SET_WORD(addr, v) do {\
 	asm les bx,zmp;\
 	asm add bx,addr;\
@@ -482,9 +556,16 @@ extern zbyte *zmp;
 	mov word ptr pcp,bx;\
 	mov word ptr pcp+2,ax }
 
+#endif /* !__WATCOMC__ */
+
 #endif /* MSDOS_16BIT */
+/******************************************************************************/
+/******************************************************************************/
 
 
+/******************************************************************************/
+/* Macros for neither Amiga nor MSDOS... that is Unix et al. */
+/******************************************************************************/
 #if !defined (AMIGA) && !defined (MSDOS_16BIT)
 
 extern zbyte *pcp;
@@ -501,6 +582,8 @@ extern zbyte *zmp;
 #define SET_PC(v)         { pcp = zmp + v; }
 
 #endif
+/******************************************************************************/
+/******************************************************************************/
 
 
 /*** Various data ***/
@@ -739,6 +822,7 @@ void	print_char(zchar);
 void	print_num(zword);
 void	print_object(zword);
 void 	print_string(const char *);
+void	reset_window(void);
 
 void 	stream_mssg_on(void);
 void 	stream_mssg_off(void);
@@ -762,11 +846,7 @@ int colour_in_use(zword);
 
 void screen_new_line(void);
 
-#ifndef MSDOS_16BIT
-#define cdecl
-#endif
-
-int cdecl zgetopt(int, char **, const char *);
+int zgetopt(int, char **, const char *);
 
 /*** returns the current window ***/
 Zwindow * curwinrec(void);
