@@ -62,13 +62,14 @@ static void sigwinch_handler(int);
 #define INFORMATION "\
 An interpreter for all Infocom and other Z-Machine games.\n\
 \n\
-Syntax: frotz [options] story-file\n\
-  -a   watch attribute setting    \t -O   watch object locating\n\
-  -A   watch attribute testing    \t -p   plain ASCII output only\n\
-  -b <colorname> background color \t -P   alter piracy opcode\n\
-  -c # context lines              \t -q   quiet (disable sound effects)\n\
-  -d   disable color              \t -r # right margin\n\
-  -e   enable sound               \t -R <path> restricted read/write\n\
+Syntax: frotz [options] story-file [blorb file]\n\
+  -a   watch attribute setting    \t -o   watch object movement\n\
+  -A   watch attribute testing    \t -O   watch object locating\n\
+  -b <colorname> background color \t -p   plain ASCII output only\n\
+  -c # context lines              \t -P   alter piracy opcode\n\
+  -d   disable color              \t -q   quiet (disable sound effects)\n\
+  -e   enable sound               \t -r # right margin\n\
+  -E <style> emphasis style       \t -R <path> restricted read/write\n\
   -f <colorname> foreground color \t -s # random number seed value\n\
   -F   Force color mode           \t -S # transcript width\n\
   -h # text height                \t -t   set Tandy bit\n\
@@ -76,7 +77,7 @@ Syntax: frotz [options] story-file\n\
   -I # interpreter number         \t -v   show version information\n\
   -l # left margin                \t -w # text width\n\
   -L <file> load this save file   \t -x   expand abbreviations g/x/z\n\
-  -o   watch object movement      \t -Z # error checking (see below)\n"
+  -m   enable mouse support       \t -Z # error checking (see below)\n"
 
 #define INFO2 "\
 Error checking: 0 none, 1 first only (default), 2 all, 3 exit after any error.\n\
@@ -98,6 +99,7 @@ static void	usage(void);
 static void	print_version(void);
 static int	getconfig(char *);
 static int	getbool(char *);
+static int	getemph(char *);
 static int	getcolor(char *);
 static int	geterrmode(char *);
 static FILE	*pathopen(const char *, const char *, const char *);
@@ -196,11 +198,6 @@ void os_process_arguments (int argc, char *argv[])
 {
 	int c;
 	char *p = NULL;
-/*
- * FIXME: Remove this after K&R treatment
- * FIXME: put this back before committing merge fixes
- *    char *blorb_ext = NULL;
- */
 	char *home;
 	char configfile[FILENAME_MAX + 1];
 
@@ -265,7 +262,7 @@ void os_process_arguments (int argc, char *argv[])
 
 	/* Parse the options */
 	do {
-		c = zgetopt(argc, argv, "aAb:c:def:Fh:iI:l:L:oOpPqr:R:s:S:tu:vw:W:xZ:");
+		c = zgetopt(argc, argv, "aAb:c:deE:f:Fh:iI:l:L:moOpPqr:R:s:S:tu:vw:W:xZ:");
 		switch(c) {
 		case 'a': f_setup.attribute_assignment = 1; break;
 		case 'A': f_setup.attribute_testing = 1; break;
@@ -280,6 +277,8 @@ void os_process_arguments (int argc, char *argv[])
 		case 'c': f_setup.context_lines = atoi(zoptarg); break;
 		case 'd': u_setup.disable_color = 1; break;
 		case 'e': f_setup.sound = 1; break;
+		case 'E':
+			u_setup.emphasis_mode = getemph(zoptarg); break;
 		case 'f':
 			u_setup.foreground_color = getcolor(zoptarg);
 			u_setup.force_color = 1;
@@ -300,6 +299,7 @@ void os_process_arguments (int argc, char *argv[])
 			f_setup.restore_mode = 1;
 			f_setup.tmp_save_name = strdup(zoptarg);
 			break;
+		case 'm': u_setup.mouse_enabled = TRUE; break;
 		case 'o': f_setup.object_movement = 1; break;
 		case 'O': f_setup.object_locating = 1; break;
 		case 'p': u_setup.plain_ascii = 1; break;
@@ -352,25 +352,25 @@ void os_process_arguments (int argc, char *argv[])
 		*p = '\0';  /* extension removed */
 
 	/* Create nice default file names */
-	f_setup.script_name = malloc((strlen(f_setup.story_name) + strlen(EXT_SCRIPT)) * sizeof(char) + 1);
+	f_setup.script_name = malloc((strlen(f_setup.story_name) + strlen(EXT_SCRIPT) + 1) * sizeof(char));
 	memcpy(f_setup.script_name, f_setup.story_name, (strlen(f_setup.story_name) + strlen(EXT_SCRIPT)) * sizeof(char));
-	strncat(f_setup.script_name, EXT_SCRIPT, strlen(EXT_SCRIPT) + 1);
+	strncat(f_setup.script_name, EXT_SCRIPT, strlen(EXT_SCRIPT)+1);
 
-	f_setup.command_name = malloc((strlen(f_setup.story_name) + strlen(EXT_COMMAND)) * sizeof(char) + 1);
+	f_setup.command_name = malloc((strlen(f_setup.story_name) + strlen(EXT_COMMAND) + 1) * sizeof(char));
 	memcpy(f_setup.command_name, f_setup.story_name, (strlen(f_setup.story_name) + strlen(EXT_COMMAND)) * sizeof(char));
-	strncat(f_setup.command_name, EXT_COMMAND, strlen(EXT_COMMAND) + 1);
+	strncat(f_setup.command_name, EXT_COMMAND, strlen(EXT_COMMAND)+1);
 
 	if (!f_setup.restore_mode) {
-		f_setup.save_name = malloc((strlen(f_setup.story_name) + strlen(EXT_SAVE)) * sizeof(char) + 1);
+		f_setup.save_name = malloc((strlen(f_setup.story_name) + strlen(EXT_SAVE) + 1) * sizeof(char));
 		memcpy(f_setup.save_name, f_setup.story_name, (strlen(f_setup.story_name) + strlen(EXT_SAVE)) * sizeof(char));
 		strncat(f_setup.save_name, EXT_SAVE, strlen(EXT_SAVE) + 1);
 	} else {  /*Set our auto load save as the name_save*/
-		f_setup.save_name = malloc((strlen(f_setup.tmp_save_name) + strlen(EXT_SAVE)) * sizeof(char) + 1);
-		memcpy(f_setup.save_name, f_setup.tmp_save_name, (strlen(f_setup.tmp_save_name) + strlen(EXT_SAVE)) * sizeof(char));
+		f_setup.save_name = malloc((strlen(f_setup.tmp_save_name) + strlen(EXT_SAVE) + 1) * sizeof(char));
+		memcpy(f_setup.save_name, f_setup.tmp_save_name, (strlen(f_setup.story_name) + strlen(EXT_SAVE)) * sizeof(char));
 		free(f_setup.tmp_save_name);
 	}
 
-	f_setup.aux_name = malloc((strlen(f_setup.story_name) + strlen(EXT_AUX)) * sizeof(char) + 1);
+	f_setup.aux_name = malloc((strlen(f_setup.story_name) + strlen(EXT_AUX) + 1) * sizeof(char));
 	memcpy(f_setup.aux_name, f_setup.story_name, (strlen(f_setup.story_name) + strlen(EXT_AUX)) * sizeof(char));
 	strncat(f_setup.aux_name, EXT_AUX, strlen(EXT_AUX) + 1);
 
@@ -464,17 +464,33 @@ void os_init_screen (void)
 	keypad(stdscr, TRUE);		/* Enable the keypad and function keys */
 	scrollok(stdscr, FALSE);	/* No scrolling unless explicitly asked for */
 
+	/* It's nice to know what terminal we have to work with. */
+	u_setup.term = strdup(getenv("TERM"));
+
 	if (z_header.version == V3 && u_setup.tandy_bit != 0)
 		z_header.config |= CONFIG_TANDY;
 
 	if (z_header.version == V3)
 		z_header.config |= CONFIG_SPLITSCREEN;
 
-	if (z_header.version >= V4)
-		z_header.config |= CONFIG_BOLDFACE | CONFIG_EMPHASIS | CONFIG_FIXED | CONFIG_TIMEDINPUT;
+	if (z_header.version >= V4) {
+		z_header.config |= CONFIG_BOLDFACE | CONFIG_FIXED | CONFIG_TIMEDINPUT;
+		if (u_setup.emphasis_mode != EMPHASIS_NONE)
+			z_header.config |= CONFIG_EMPHASIS;
+	}
 
 	if (z_header.version >= V5)
-		z_header.flags &= ~(GRAPHICS_FLAG | MOUSE_FLAG | MENU_FLAG);
+		z_header.flags &= ~(GRAPHICS_FLAG | MENU_FLAG);
+
+#ifdef NCURSES_MOUSE_VERSION
+	/* Limited mouse support */
+	if (u_setup.mouse_enabled && z_header.version >= V5)
+		mousemask(BUTTON1_CLICKED | BUTTON1_DOUBLE_CLICKED, NULL);
+	else
+#endif
+	if (z_header.version >= V5) {
+		z_header.flags &= ~MOUSE_FLAG;
+	}
 
 #ifdef NO_SOUND
 	if (z_header.version >= V5)
@@ -892,6 +908,9 @@ static int getconfig(char *configfile)
 		else if (strcmp(varname, "force_color") == 0) {
 			u_setup.force_color = getbool(value);
 		}
+		else if (strcmp(varname, "emphasis_mode") == 0) {
+			u_setup.emphasis_mode = getemph(value);
+		}
 		else if (strcmp(varname, "obj_move") == 0) {
 			f_setup.object_movement = getbool(value);
 		}
@@ -962,6 +981,36 @@ static int getconfig(char *configfile)
 } /* getconfig */
 
 
+/* getemph
+ *
+ * Check a string for possible emphasis types:
+ * 	italic, underline, caps, or none
+ *
+ */
+static int getemph(char *value)
+{
+	int num;
+
+	/* Be case-insensitive */
+	for (num = 0; value[num] != 0; num++)
+		value[num] = tolower((int) value[num]);
+
+	if (strncmp(value, "italic", 6) == 0)
+		return EMPHASIS_ITALIC;
+
+	if (strncmp(value, "under", 5) == 0)
+		return EMPHASIS_UNDERLINE;
+
+	if (strncmp(value, "none", 2) == 0)
+		return EMPHASIS_NONE;
+
+	if (atoi(value) == 0)
+		return EMPHASIS_NONE;
+
+	return EMPHASIS_ITALIC;
+}
+
+
 /*
  * getbool
  *
@@ -1102,6 +1151,7 @@ void os_init_setup(void)
 	u_setup.force_color = 0;
 	u_setup.foreground_color = -1;
 	u_setup.background_color = -1;
+	u_setup.emphasis_mode = EMPHASIS_DEFAULT;
 	u_setup.screen_width = -1;
 	u_setup.screen_height = -1;
 	u_setup.random_seed = -1;
@@ -1118,6 +1168,7 @@ void os_init_setup(void)
 	/* u_setup.interpreter = INTERP_DEFAULT; */
 	u_setup.current_color = 0;
 	u_setup.color_enabled = FALSE;
+	u_setup.mouse_enabled = FALSE;
 } /* os_init_setup */
 
 
@@ -1168,6 +1219,9 @@ static void print_version(void)
 	printf("  Frotz was originally written by Stefan Jokisch.\n");
 	printf("  It complies with standard 1.0 of Graham Nelson's specification.\n");
 	printf("  It was ported to Unix by Galen Hazelwood.\n");
+	printf("  It is distributed under the GNU General Public License version 2 or\n");
+	printf("    (at your option) any later version.\n");
+	printf("  This software is offered as-is with no warranty or liability.\n");
 	printf("  The core and Unix port are maintained by David Griffith.\n");
 	printf("  Frotz's homepage is https://661.org/proj/if/frotz/\n\n");
 	return;
