@@ -139,17 +139,19 @@ extern char m_names_format;
 static char user_names_format = 0;
 extern char *m_reslist_file;
 extern int option_scrollback_buffer;
+extern bool option_disable_color;
 
 static char *info_header =
     "FROTZ V%s - SDL graphics and audio interface.\n"
     "An interpreter for all Infocom and other Z-Machine games.\n\n"
-    "Syntax: sfrotz [options] story-file\n";
+    "Syntax: sfrotz [options] story-file [blorb file]\n";
 
 static char *info[] = {
 	"-a   watch attribute setting",
 	"-A   watch attribute testing",
 	"-b <colourname> background colour",
 	"-c # context lines",
+	"-d   disable color",
 	"-f <colorname> foreground colour",
 	"-F   fullscreen mode",
 	"-H # screen height",
@@ -253,7 +255,7 @@ extern int optind;
 
 extern int m_timerinterval;
 
-static char *options = "@:%aAb:B:c:f:FH:iI:l:L:m:N:oOPqr:s:S:tTu:vVW:xXZ:";
+static char *options = "@:%aAb:B:c:df:FH:iI:l:L:m:N:oOPqr:s:S:tTu:vVW:xXZ:";
 
 static int limit(int v, int m, int M)
 {
@@ -291,6 +293,8 @@ static void parse_options(int argc, char **argv)
 			option_scrollback_buffer = num;
 		if (c == 'c')
 			f_setup.context_lines = num;
+		if (c == 'd')
+			option_disable_color = 1;
 		if (c == 'm')
 			m_timerinterval = limit(num, 10, 1000000);
 		if (c == 'N')
@@ -305,8 +309,10 @@ static void parse_options(int argc, char **argv)
 			user_screen_height = num;
 		if (c == 'i')
 			f_setup.ignore_errors = 1;
-		if (c == 'I')
+		if (c == 'I') {
 			f_setup.interpreter_number = num;
+			z_header.interpreter_number = num;
+		}
 		if (c == 'l')
 			f_setup.left_margin = num;
 		if (c == 'L') {
@@ -363,11 +369,12 @@ static void print_version(void)
 	printf("Git commit:     %s\n", GIT_HASH);
 	printf("Notes:          %s\n", RELEASE_NOTES);
 	printf("  Frotz was originally written by Stefan Jokisch.\n");
-	printf
-	    ("  It complies with standard 1.0 of Graham Nelson's specification.\n");
+	printf("  It complies with standard 1.0 of Graham Nelson's specification.\n");
 	printf("  It was ported to Unix by Galen Hazelwood.\n");
-	printf
-	    ("  The core and SDL port are maintained by David Griffith.\n");
+	printf("  It is distributed under the GNU General Public License version 2 or\n");
+	printf("    (at your option) any later version.\n");
+	printf("  This software is offered as-is with no warranty or liability.\n");
+	printf("  The core and SDL port are maintained by David Griffith.\n");
 	printf("  Frotz's homepage is https://661.org/proj/if/frotz/\n\n");
 	os_quit(EXIT_SUCCESS);
 }
@@ -447,36 +454,26 @@ void os_process_arguments(int argc, char *argv[])
 	f_setup.story_path = new_dirname(argv[zoptind]);
 
 	/* Create nice default file names */
-	f_setup.script_name =
-	    malloc((strlen(f_setup.story_name) +
-		    strlen(EXT_SCRIPT)) * sizeof(char) + 1);
-	memcpy(f_setup.script_name, f_setup.story_name, strlen(f_setup.story_name) * sizeof(char));
-	strncat(f_setup.script_name, EXT_SCRIPT, strlen(EXT_SCRIPT) + 1);
+	f_setup.script_name = malloc((strlen(f_setup.story_name) + strlen(EXT_SCRIPT) + 1) * sizeof(char));
+	memcpy(f_setup.script_name, f_setup.story_name, (strlen(f_setup.story_name) + strlen(EXT_SCRIPT)) * sizeof(char));
+	strncat(f_setup.script_name, EXT_SCRIPT, strlen(EXT_SCRIPT)+1);
 
-	f_setup.command_name =
-	    malloc((strlen(f_setup.story_name) +
-		    strlen(EXT_COMMAND)) * sizeof(char) + 1);
-	memcpy(f_setup.command_name, f_setup.story_name, strlen(f_setup.story_name) * sizeof(char));
-	strncat(f_setup.command_name, EXT_COMMAND, strlen(EXT_COMMAND) + 1);
+	f_setup.command_name = malloc((strlen(f_setup.story_name) + strlen(EXT_COMMAND) + 1) * sizeof(char));
+	memcpy(f_setup.command_name, f_setup.story_name, (strlen(f_setup.story_name) + strlen(EXT_COMMAND)) * sizeof(char));
+	strncat(f_setup.command_name, EXT_COMMAND, strlen(EXT_COMMAND)+1);
 
 	if (!f_setup.restore_mode) {
-		f_setup.save_name =
-		    malloc((strlen(f_setup.story_name) +
-			    strlen(EXT_SAVE)) * sizeof(char) + 1);
-		memcpy(f_setup.save_name, f_setup.story_name, strlen(f_setup.story_name) * sizeof(char));
+		f_setup.save_name = malloc((strlen(f_setup.story_name) + strlen(EXT_SAVE) + 1) * sizeof(char));
+		memcpy(f_setup.save_name, f_setup.story_name, (strlen(f_setup.story_name) + strlen(EXT_SAVE)) * sizeof(char));
 		strncat(f_setup.save_name, EXT_SAVE, strlen(EXT_SAVE) + 1);
 	} else {	/* Set our auto load save as the name_save */
-		f_setup.save_name =
-		    malloc((strlen(f_setup.tmp_save_name) +
-			    strlen(EXT_SAVE)) * sizeof(char) + 1);
-		memcpy(f_setup.save_name, f_setup.tmp_save_name, strlen(f_setup.tmp_save_name) * sizeof(char));
+		f_setup.save_name = malloc((strlen(f_setup.tmp_save_name) + strlen(EXT_SAVE) + 1) * sizeof(char));
+		memcpy(f_setup.save_name, f_setup.tmp_save_name, (strlen(f_setup.story_name) + strlen(EXT_SAVE)) * sizeof(char));
 		free(f_setup.tmp_save_name);
 	}
 
-	f_setup.aux_name =
-	    malloc((strlen(f_setup.story_name) +
-		    strlen(EXT_AUX)) * sizeof(char) + 1);
-	memcpy(f_setup.aux_name, f_setup.story_name, strlen(f_setup.story_name) * sizeof(char));
+	f_setup.aux_name = malloc((strlen(f_setup.story_name) + strlen(EXT_AUX) + 1) * sizeof(char));
+	memcpy(f_setup.aux_name, f_setup.story_name, (strlen(f_setup.story_name) + strlen(EXT_AUX)) * sizeof(char));
 	strncat(f_setup.aux_name, EXT_AUX, strlen(EXT_AUX) + 1);
 
 	/* Save the executable file name */
